require 'rest_client'
require 'json'

module Heroix
  Unauthorized = RestClient::Unauthorized

  class Resource
    attr_reader :url, :token, :type

    def initialize(params)
      @url = params[:url]
      @token = params[:token]
      @type = params[:type]
    end

    def with_token(value)
      @token = value
      self
    end

    def index(params = {})
      request(:get, url, :query => params)
    end

    def show(id)
      request(:get, url_for(id))
    end

    def create(data)
      request(:post, url, :data => data)
    end

    def update(id, data)
      request(:put, url_for(id), :data => data)
    end

    def delete(id)
      request(:delete, url_for(id))
    end

    protected

    def url_for(path)
      "#{url}/#{path}"
    end

    private

    def request(method, url, options = {})
      rest_logger = RestClient.log
      RestClient.log = logger

      perform_request(method, url, options)
    ensure
      RestClient.log = rest_logger
    end

    def perform_request(method, url, options)
      handle RestClient::Request.execute(
        :method => method,
        :url => url,
        :payload => prepare_data(options[:data]),
        :headers => {
          :authorization => "Token token=\"#{token}\"",
          :content_type => :json, :params => options[:query]
        }
      )
    end

    def handle(response)
      deprecation = response.headers[:x_deprecation]
      logger.warn "[heroix] DEPRECATION WARNING #{deprecation}" if deprecation

      parse_response response.to_str
    end

    def parse_response(response)
      return {} if response.empty?
      JSON.parse(response)
    end

    def prepare_data(data)
      data = { type => data } if data && type
      data.to_json
    end

    def logger
      Heroix.logger
    end
  end
end
